//
//  CMRBrowser-Validation.m
//  BathyScaphe
//
//  Updated by Tsutomu Sawada on 09/09/04.
//  Copyright 2005-2010 BathyScaphe Project. All rights reserved.
//  encoding="UTF-8"
//

#import "CMRBrowser_p.h"
#import "SmartBoardList.h"
#import "missing.h"

@implementation CMRBrowser(Validation)
- (BOOL)segCtrlTbItem:(BSSegmentedControlTbItem *)item validateSegment:(int)segment
{
	if ([[item itemIdentifier] isEqualToString:@"Toggle View Mode"]) {
		return [[self currentThreadsList] viewMode] < 2;
	}
	return [super segCtrlTbItem:item validateSegment:segment];
}

#pragma mark Validation Helpers
- (BOOL)isIndexFieldFirstResponder
{
	NSWindow *window = [self window];
	if (![[window firstResponder] isKindOfClass:[NSTextView class]]) {
        return NO;
    }
	id fieldEditor = [window fieldEditor:NO forObject:nil];
	if (!fieldEditor) {
        return NO;
    }
	return ([[(NSTextView *)[window firstResponder] delegate] isKindOfClass:[IndexField class]]);
}

- (BOOL)validateBBSActionItems
{
    NSView *focusedView_ = (NSView *)[[self window] firstResponder];
    if (focusedView_ == [self textView] || [self isIndexFieldFirstResponder]) {
        return ([self threadAttributes] != nil);
    } else if (focusedView_ = [self threadsListTable]) {
        return [[self currentThreadsList] isBoard];
    } else if (focusedView_ = [self boardListTable]) {
        int selectedRow = [[self boardListTable] selectedRow];
        if (selectedRow != -1) {
            id item = [[self boardListTable] itemAtRow:selectedRow];
            return [BoardListItem isBoardItem:(BoardListItem *)item];
        }
    }
    return NO;
}

- (BOOL)validateDeleteThreadItemsEnabling:(NSArray *)threads
{
	NSEnumerator		*iter_;
	NSDictionary		*thread_;

	if (!threads || [threads count] == 0) {
        return NO;
	}

	iter_ = [threads objectEnumerator];
	while (thread_ = [iter_ nextObject]){
		NSNumber	*status_;
		
		status_ = [thread_ objectForKey:CMRThreadStatusKey];
		if (!status_) {
            continue;
		}
		if (ThreadLogCachedStatus & [status_ unsignedIntValue]) {
			return YES;
		}
	}

	return NO;
}

- (BOOL)validateReloadThreadsListItem:(id)theItem
{
	static NSString *kToolTipNormal = nil;
	static NSString *kToolTipWaiting = nil;

	id tmp_ = [self currentThreadsList];
	if (!tmp_) {
        return NO;
    }
	if (!kToolTipNormal || !kToolTipWaiting) {
		kToolTipNormal = [NSLocalizedStringFromTable(@"Reload List ToolTip", @"ToolbarItems", @"") retain];
		kToolTipWaiting = [NSLocalizedStringFromTable(@"Reload List ToolTip 2", @"ToolbarItems", @"") retain];
	}

	if ([tmp_ isFavorites] || [tmp_ isSmartItem]) {
		BOOL canCheck = [CMRPref canHEADCheck];
		if ([theItem respondsToSelector:@selector(setToolTip:)]) {
			[theItem setToolTip:(canCheck ? kToolTipNormal : kToolTipWaiting)];
		}
		return canCheck;
	} else {
		if ([theItem respondsToSelector:@selector(setToolTip:)]) {
			[theItem setToolTip:kToolTipNormal];
		}
		return YES;
	}
}

- (BOOL)validateCollapseOrExpandBoardListItem:(id)theItem
{
	if ([theItem isKindOfClass:[NSMenuItem class]]) {
		[theItem setTitle:([[self boardListSubView] isCollapsed] ? NSLocalizedString(@"Expand Boards List", @"")
                                                                 : NSLocalizedString(@"Collapse Boards List", @""))];
	}
	return YES;
}

- (BOOL)validateDeleteBoardFromListItem:(int)tag_
{
	NSOutlineView	*bLT = [self boardListTable];
	int numOfSelectedRow = [bLT numberOfSelectedRows];
	switch (numOfSelectedRow) {
		case 0:
			return NO;
		case 1:
			return (![SmartBoardList isFavorites:[bLT itemAtRow:[bLT selectedRow]]]);
		default:
			return (tag_ == kBLDeleteItemViaMenubarItemTag);
	}
}

- (BOOL)validateBoardListContextualMenuItem:(id)menuItem
{
	BSBoardListView *view = (BSBoardListView *)[self boardListTable];
	int			rowAtMenu = [view semiSelectedRow];

	if (rowAtMenu == -1) {
		return NO;
	} else {
		NSIndexSet *selectedRows = [view selectedRowIndexes];
		if ([selectedRows count] > 1 && [selectedRows containsIndex:rowAtMenu]) {
			return ([menuItem tag] == kBLDeleteItemViaContMenuItemTag);
		}
		id item = [view itemAtRow:rowAtMenu];
		if (!item) {
            return NO;
        }
        int tag = [menuItem tag];
        if (tag == kBLShowInspectorViaContMenuItemTag) {
            [self validateShowBoardInspectorPanelItemTitle:menuItem];
            return [BoardListItem isBoardItem:item];
        } else if (tag == kBLShowLocalRulesViaContMenuItemTag || tag == kBLOpenBoardItemViaContMenuItemTag) {
            return [BoardListItem isBoardItem:item];
        } else {
            return ![BoardListItem isFavoriteItem:item];
        }
	}
	return NO;
}

- (BOOL)validateActionButtonMenuItem:(id)menuItem
{
	BSBoardListView	*view = (BSBoardListView *)[self boardListTable];
	int selectedRow = [view selectedRow];
    if (selectedRow == -1) {
        return NO;
    }
    if ([menuItem tag] == kBLShowInspectorViaActionButtonTag) {
        [self validateShowBoardInspectorPanelItemTitle:menuItem];
    }
    id item = [view itemAtRow:selectedRow];
    return (item && [BoardListItem isBoardItem:item]);
}

static BOOL isThreadContextualMenu(id item)
{
    return ([item isKindOfClass:[NSMenuItem class]] && ![[(NSMenuItem *)item menu] supermenu]);
}

#pragma mark NSUserInterfaceValidations Protocol
- (BOOL)validateUserInterfaceItem:(id <NSValidatedUserInterfaceItem>)theItem
{
	int tag_ = [theItem tag];

	if (tag_ == kBrowserMenuItemAlwaysEnabledTag) {
        return YES;
    }
	if (tag_ == kBLEditItemViaMenubarItemTag || tag_ == kBLDeleteItemViaMenubarItemTag) {
		return [self validateDeleteBoardFromListItem:tag_];
	}
    if (tag_ == kBLOpenBoardItemViaActionButtonTag || tag_ == kBLShowInspectorViaActionButtonTag || tag_ == kBLShowLocalRulesViaActionButtonTag) {
        return [self validateActionButtonMenuItem:theItem];
    }
	if ((tag_ > kBLContMenuItemTagMin) && (tag_ < kBLContMenuItemTagMax)) {
		return [self validateBoardListContextualMenuItem:theItem];
	}

	SEL action_ = [theItem action];

	if (action_ == @selector(showSearchThreadPanel:) || action_ == @selector(chooseColumn:)) {
		return ([self currentThreadsList] != nil);
	} else if (action_ == @selector(saveAsDefaultFrame:)) {
		return NO;
	} else if (action_ == @selector(openSelectedThreads:)) {
		NSArray *tmp = [self selectedThreadsReallySelected];
		if (!tmp) {
            return NO;
        }
		if ([tmp count] == 1 && [self shouldShowContents] && [[[tmp lastObject] objectForKey:CMRThreadLogFilepathKey] isEqualToString:[self path]]) {
			return NO;
		}
		return YES;
	} else if (action_ == @selector(collapseOrExpandBoardList:)) {
		return [self validateCollapseOrExpandBoardListItem:theItem];
	} else if (action_ == @selector(reloadThreadsList:)) {
		return [self validateReloadThreadsListItem:theItem];
	}

	if (action_ == @selector(addFavorites:)) {
		CMRFavoritesOperation	operation_;

		if (tag_ == 781) { // Browser Contextual Menu
			operation_ = [self favoritesOperationForThreads:[self selectedThreadsReallySelected]];
		} else {
			if(isThreadContextualMenu(theItem)) { // Thread Contextual Menu
				operation_ = [[CMRFavoritesManager defaultManager] availableOperationWithPath:[self path]];
			} else {
				NSView *focusedView_ = (NSView *)[[self window] firstResponder];
				if (focusedView_ == [self textView] || [self isIndexFieldFirstResponder]) {
					operation_ = [[CMRFavoritesManager defaultManager] availableOperationWithPath:[self path]];
				} else {
					operation_ = [self favoritesOperationForThreads:[self selectedThreadsReallySelected]];
				}
			}
		}
		return [self validateAddFavoritesItem:theItem forOperation:operation_];
	}

	if(action_ == @selector(deleteThread:)) {

		[self validateDeleteThreadItemTitle:theItem];
		
		if (tag_ == 780) { // Browser Contextual Menu
			return [self validateDeleteThreadItemsEnabling:[self selectedThreadsReallySelected]];
		} else {
			if (isThreadContextualMenu(theItem)) { // Thread Contexual Menu
				return [super validateDeleteThreadItemEnabling:[self path]];
			} else {
				NSView *focusedView_ = (NSView *)[[self window] firstResponder];
				if (focusedView_ == [self textView] || [self isIndexFieldFirstResponder]) {
					return [super validateDeleteThreadItemEnabling:[self path]];
				} else {
					return [self validateDeleteThreadItemsEnabling:[self selectedThreadsReallySelected]];
				}
			}
		}
		
		return NO;
	}
	
	if (action_ == @selector(reloadThread:)) {
		if (tag_ == 782) { // browser contextual menu
			return YES;
		} else {
			if (isThreadContextualMenu(theItem)) { // Thread Contextual Menu
				return [self threadAttributes] && ![[self document] isDatOchiThread];
			} else {
                NSView *focusedView_ = (NSView *)[[self window] firstResponder];
				if (focusedView_ == [self textView] || [self isIndexFieldFirstResponder]) {
					return [self threadAttributes] && ![[self document] isDatOchiThread];
				} else {
					return ([[self selectedThreadsReallySelected] count] > 0);
				}
			}
		}
		
		return NO;
	}

    if (action_ == @selector(retrieveThread:)) {
        NSView *focusedView_ = (NSView *)[[self window] firstResponder];
        if (focusedView_ == [self textView] || [self isIndexFieldFirstResponder]) {
            return [super validateUserInterfaceItem:theItem];
        } else {
            return ([[self selectedThreadsReallySelected] count] > 0);
        }
    }

    if (action_ == @selector(toggleLayout:)) {
        int current = [[self layoutSwitcher] selectedSegment];
        if (current == [theItem tag]) {
            [(NSMenuItem *)theItem setState:NSOnState];
        } else {
            [(NSMenuItem *)theItem setState:NSOffState];
        }
        return YES;
    }
	
	return [super validateUserInterfaceItem:theItem];
}
@end
